-- **************************************************************************
--  This is FIR filter with Floating process(I/Q)
--				2006,11,05 by Y nishimura
--
--		Narrow mode  1/2 decimation
--			Band 01: 0-5.7KHz  6.0KHz   x1/2  wight 1/10
--			Band 10: 0-2.7KHz  3.0KHz   x1    wight 1/10
-- **************************************************************************
LIBRARY ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
LIBRARY lpm;
use lpm.lpm_components.all;

entity FirFront is
	port
		(
			Hclk		: in  std_logic;	-- Master 4fsc clock
			Reset		: in  std_logic;	-- Master Reset
			CWkey		: in  std_logic;	-- CW key input
			CWmode		: in  std_logic;	-- CW mode
			Cgate		: in  std_logic;	-- 1/1024
			Ifband		: in  std_logic; 	-- 1:7K  0:3K
			IdatIn		: in  std_logic_vector(30 downto 0);
			Dgate		: out std_logic;
			DaOutI		: out std_logic_vector(30 downto 0)
		);
end FirFront;

architecture norm of FirFront is
	signal	AcchI				: std_logic_vector(38 downto 0);
	signal	MuloI				: std_logic_vector(35 downto 0);
	signal	AcclI				: std_logic_vector(33 downto 0);
	signal	OutIq				: std_logic_vector(31 downto 0);
	signal	OutIr				: std_logic_vector(31 downto 0);
	signal	OutI				: std_logic_vector(30 downto 0);
	signal	Fdaa,Coeff			: std_logic_vector(17 downto 0);
	signal	CWda				: std_logic_vector(17 downto 0);
	signal	FirIna,FIna,FirInc	: std_logic_vector(17 downto 0);
	signal	CoeffD,CoeffF		: std_logic_vector(17 downto 0);
	signal	LIMs,LQMs			: std_logic_vector(15 downto 0);
	signal	Roma				: std_logic_vector(9 downto 0);
	signal	CWadd				: std_logic_vector(8 downto 0);
	signal	Wadd,Radd			: std_logic_vector(8 downto 0);
	signal	Stat,Lda,Wsta 		: std_logic_vector(1 downto 0);
	signal	Stime,Str,StrS,Lal	: std_logic;
	signal	Aload,Dload,Ldel	: std_logic;
	signal	Bload,Oload,Dire	: std_logic;

begin
	DaOutI <= OutI;
------------------------------------------------------------------------------
--		CW wave form
------------------------------------------------------------------------------
	U17: LPM_ROM generic map(
				 LPM_WIDTH => 18, LPM_WIDTHAD => 9,
                 LPM_ADDRESS_CONTROL => "REGISTERED",
				 LPM_OUTDATA => "REGISTERED",
				 LPM_FILE => "CWtran.MIF")
        port map(
					ADDRESS => CWadd,
                    INCLOCK => Hclk,
                    OUTCLOCK => Hclk,
					MEMENAB => '1',
					Q => CWda
				);

	process(Hclk,Reset)
	begin
		if Reset = '0' 		then CWadd <= "000000000"; Dire <= '0';
		elsif rising_edge(Hclk) then
			if Cgate = '1'	then
				if Dire = '1'	then
					if CWadd = "111111111"	then CWadd <= CWadd; Dire <= CWkey;
											else CWadd <= CWadd + 1; Dire <= Dire;
					end if;
				else
					if CWadd = "000000000"	then CWadd <= CWadd; Dire <= CWkey;
											else CWadd <= CWadd - 1; Dire <= Dire;
					end if;
				end if;
			else CWadd <= CWadd; Dire <= Dire;
			end if;
		end if;
	end process;
	
------------------------------------------------------------------------------
--		Status counter
------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' 		then Stat <= "00";
		elsif rising_edge(Hclk) then
			if Cgate = '1'	then Stat <= "00";
							else Stat <= Stat + 1;
			end if;
		end if;
	end process;
	
-------------------------------------------------------------------------------
--		Memory Write address
-------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' 		then Wadd <= "000000000"; Wsta <= "00";
		elsif rising_edge(Hclk) then
			case Wsta is
				when "00" => if Cgate = '1'	then Wsta <= "01";
											else Wsta <= "00";
							 end if;
				when "01" => if Cgate = '0'	then Wsta <= "11";
											else Wsta <= "01";
							 end if;
				when "10" => Wsta <= "00";
				when "11" => Wsta <= "10";
				when others => Wsta <= "00";
			end case;
			if Wsta = "10"	then Wadd <= Wadd + 1;
							else Wadd <= Wadd;
			end if;
		end if;
	end process;
	
	process(Hclk,Reset)
	begin
		if Reset = '0' 		then Stime <= '0';
		elsif rising_edge(Hclk) then
			if Cgate = '1'	then Stime <= '1';
							else Stime <= '0';
			end if;
		end if;
	end process;

	process(Hclk,Reset)
	begin
		if Reset = '0' 		then
			Lda <= "00"; Str <= '0'; StrS <= '0';
		elsif rising_edge(Hclk) then
			if Lda = "10"	then Str <= '1';
							else Str <= '0';
			end if;
			StrS <= Str;
			case Lda is
				when "00" => if Stime = '1'	then Lda <= "01";
											else Lda <= "00";
							 end if;
				when "01" => Lda <= "11";
				when "10" => Lda <= "00";
				when "11" => Lda <= "10";
				when others => Lda <= "00";
			end case;
		end if;
	end process;

-------------------------------------------------------------------------------
--		Memory read
-------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' 		then
			Radd <= "000000000"; Roma <= "0000000000";
		elsif rising_edge(Hclk) then
			if Stime = '1'		then Radd <= Wadd;
			else
				if Stat(0) = '0' then 	
					 Radd <= Radd - 1;
				else Radd <= Radd;
				end if;
			end if;
			if Stime = '1'			then Roma <= "0000000000";
			else
				if Stat(0) = '0'	then Roma <= Roma + 1;
									else Roma <= Roma;
				end if;
			end if;
		end if;
	end process;

------------------------------------------------------------------------------
--		Data Save into buffer
------------------------------------------------------------------------------
	U1: lpm_ram_dp
   		GENERIC map (
						LPM_WIDTH =>18, LPM_WIDTHAD => 9,
      					LPM_OUTDATA => "REGISTERED")
		port map (
					data => IdatIn(17 downto 0),
					wraddress => Wadd,
					wrclken => '1',
					wren => Cgate,
					wrclock => Hclk,
					rdaddress => Radd,
					rden => '1',
					rdclken => '1',
					rdclock => Hclk,
					q => FirIna
				  );

	U3: lpm_ram_dp
   		GENERIC map (
						LPM_WIDTH =>18, LPM_WIDTHAD => 9,
      					LPM_OUTDATA => "REGISTERED")
		port map (
					data => FIna,
					wraddress => Wadd,
					wrclken => '1',
					wren => Cgate,
					wrclock => Hclk,
					rdaddress => Radd,
					rden => '1',
					rdclken => '1',
					rdclock => Hclk,
					q => FirInc
				  );

	FIna(12 downto 0) <= IdatIn(30 downto 18);
	FIna(17 downto 13) <= "00000";

 	process(Hclk,Reset)
	begin
		if Reset = '0' then
			Fdaa <= "000000000000000000";
		elsif rising_edge(Hclk) 	then
			if Stat(0) = '1'	then
				Fdaa(17 downto 13) <= "00000"; 
				Fdaa(12 downto 0)  <= FirIna(12 downto 0);
			else
				Fdaa(17 downto 5) <= FirInc(12 downto 0);
				Fdaa(4 downto 0) <= FirIna(17 downto 13);
			end if;
		end if;
	end process;

----------------------------------------------------------------------------------
--	Coefficient Memory
----------------------------------------------------------------------------------
	U10: LPM_ROM generic map(
				 LPM_WIDTH => 18, LPM_WIDTHAD => 9,
                 LPM_ADDRESS_CONTROL => "REGISTERED",
				 LPM_OUTDATA => "REGISTERED",
				 LPM_FILE => "F509F7k.MIF")
        port map(
					ADDRESS => Roma(8 downto 0),
                    INCLOCK => Hclk,
                    OUTCLOCK => Hclk,
					MEMENAB => '1',
					Q => CoeffD
				);

	U12: LPM_ROM generic map(
				 LPM_WIDTH => 18, LPM_WIDTHAD => 9,
                 LPM_ADDRESS_CONTROL => "REGISTERED",
				 LPM_OUTDATA => "REGISTERED",
				 LPM_FILE => "F509F3k.MIF")
        port map(
					ADDRESS => Roma(8 downto 0),
                    INCLOCK => Hclk,
                    OUTCLOCK => Hclk,
					MEMENAB => '1',
					Q => CoeffF
				);

 	process(Hclk,Reset)
	begin
		if Reset = '0' then
			Coeff <= "000000000000000000";
		elsif rising_edge(Hclk) 	then
			If Ifband = '1'	then Coeff <= CoeffD;
							else Coeff <= CoeffF;
			end if;
		end if;
	end process;

----------------------------------------------------------------------------------
--	Multiply for Sub data
----------------------------------------------------------------------------------
	U16: lpm_mult GENERIC map (
			LPM_WIDTHA => 18,LPM_WIDTHB => 18,LPM_WIDTHP => 36,
			LPM_REPRESENTATION => "SIGNED",LPM_PIPELINE => 2) 
  	    PORT map (
					dataa => Fdaa,
					datab => Coeff,
					aclr  => '0',
					clock => Hclk,
					clken => '1',
        			result => MuloI
				 ); 

------------------------------------------------------------------------------
--		Accl
------------------------------------------------------------------------------
	process(Hclk,Reset)
		variable Tempa : std_logic_vector(33 downto 0);
		
	begin
		if Reset = '0' 		then
			AcclI <= "0000000000000000000000000000000000";
		elsif rising_edge(Hclk) then
			Tempa(33) := MuloI(35); Tempa(32) := MuloI(35);
			Tempa(31 downto 0) := MuloI(35 downto 4);
			if Str = '1'					then
				AcclI <= "0000000000000000000000000000000000";
			else
				if stat(0) = '0'			then
					 AcclI <= AcclI + Tempa;
				else AcclI <= AcclI;
				end if;
			end if;
		end if;
	end process;

----------------------------------------------------------------------------------
--		Acch
----------------------------------------------------------------------------------
	process(Hclk,Reset)
		variable Tempa : std_logic_vector(38 downto 0);
		
	begin
		if Reset = '0' 		then
			AcchI <= "000000000000000000000000000000000000000";
		elsif rising_edge(Hclk) then
			Tempa(38) := MuloI(35); Tempa(37) := MuloI(35); Tempa(36) := MuloI(35);
			Tempa(35 downto 0) := MuloI;
			if StrS = '1'	then AcchI <= "000000000000000000000000000000000000000";
			else
				if stat(0) = '1'	then
					 AcchI <= AcchI + Tempa;
				else AcchI <= AcchI;
				end if;
			end if;
		end if;
	end process;

-----------------------------------------------------------------------------------
--		Output Latch
-----------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' 		then
			Aload <= '0'; Dload <= '0'; Oload <= '0';
		elsif rising_edge(Hclk) then
			Dload <= Aload;
			if Stime = '1'		then Oload <= '0';
								else Oload <= Aload or Oload;
			end if;
			if Stat(0) = '1'	then
				if Roma = "0111111111"	then Aload <= not Oload;
										else Aload <= '0';
				end if;
			else Aload <= '0';
			end if;
		end if;
	end process;
	
	process(Hclk,Reset)
	begin
		if Reset = '0' 		then Dgate <= '0'; Ldel <= '0';
		elsif rising_edge(Hclk) then
			Ldel <= Dload; Dgate <= Ldel;
		end if;
	end process;


--------------------------------------------------------------------------------
	process(Hclk,Reset)
		variable Tempa : std_logic_vector(31 downto 0);
		
	begin
		if Reset = '0' 		then OutIq <= "00000000000000000000000000000000";
		elsif rising_edge(Hclk) then
			if AcclI(33) = '0'	then Tempa(31 downto 19) := "0000000000000";
								else Tempa(31 downto 19) := "1111111111111";
			end if;
			Tempa(18 downto 0)  := AcclI(33 downto 15);
			if Aload = '1'		then OutIq <= AcchI(35 downto 4) + Tempa;
								else OutIq <= OutIq;
			end if;
		end if;
	end process;

	process(Hclk,Reset)
		variable Tempa : std_logic_vector(31 downto 0);
		
	begin
		if Reset = '0' 		then OutIr <= "00000000000000000000000000000000";
		elsif rising_edge(Hclk) then
			if AcclI(33) = '0'	then Tempa(31 downto 18) := "00000000000000";
								else Tempa(31 downto 18) := "11111111111111";
			end if;
			Tempa(17 downto 0)  := AcclI(33 downto 16);
			if Aload = '1'		then OutIr <= AcchI(36 downto 5) + Tempa;
								else OutIr <= OutIr;
			end if;
		end if;
	end process;


--------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' 		then OutI <= "0000000000000000000000000000000";
		elsif rising_edge(Hclk) then
			if Dload = '1'	then
				if CWmode = '1'		then
					OutI(30 downto 13) <= CWda; OutI(12 downto 0) <= "0000000000000";
				else
					if Ifband = '1'	then OutI <= OutIq(31 downto 1);
									else OutI <= OutIr(31 downto 1);
					end if;
				end if;
			else OutI <= OutI;
			end if;
		end if;
	end process;

end norm;
