-- **************************************************************************
--  This is 64times Oversampling filter tap 64*16
--		Coefficient are normalized in Q17 
--		Sampling rate converter   31.7kHz -> 31.5kHz
--			2014,11,11 Y. Nishimura
-- **************************************************************************
LIBRARY ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_unsigned.all;
LIBRARY lpm;
use lpm.lpm_components.all;

entity FrontOver is
	port
		(
			Hclk		: in  std_logic;	-- Master 4fsc clock
			Reset		: in  std_logic;	-- Master Reset
			Txgate		: in  std_logic;	-- Tx sampling gate 31.5kHz
			Adgate		: in  std_logic;	-- ADC gate 31.7kHz
			Indata		: in  std_logic_vector(15 downto 0); -- ADC data
			Outdata		: out std_logic_vector(15 downto 0)
		);
end FrontOver;

architecture norm of FrontOver is
	signal	Mulo,Acc			: std_logic_vector(35 downto 0);
	signal	Coeff,Fird			: std_logic_vector(17 downto 0);
	signal	Firin,Daout			: std_logic_vector(15 downto 0);
	signal	Shape				: std_logic_vector(10 downto 0);
	signal	Pcnt				: std_logic_vector(10 downto 0);
	signal	Romadd				: std_logic_vector(8 downto 0);
	signal	Spha				: std_logic_vector(5 downto 0);
	signal	Wadd,Radd,Sphb		: std_logic_vector(4 downto 0);
	signal	Fstat				: std_logic_vector(4 downto 0);
	signal	Tstart,Tend			: std_logic;

begin
	Outdata <= Daout;
------------------------------------------------------------------------------
--		Position counter
------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' 		then Pcnt <= "00000000000";
		elsif rising_edge(Hclk) then
			if Adgate = '1'	then Pcnt <= "00000000000";
							else Pcnt <= Pcnt + 1;
			end if;
		end if;
	end process;

------------------------------------------------------------------------------
--		Data buffering
------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' 		then
			Wadd <= "00000";
		elsif rising_edge(Hclk) then
			if Adgate = '1'	then Wadd <= Wadd + 1;
							else Wadd <= Wadd;
			end if;
		end if;
	end process;
					
	U1: lpm_ram_dp
   		GENERIC map (
						LPM_WIDTH =>16, LPM_WIDTHAD => 5,
      					LPM_OUTDATA => "REGISTERED")
		port map (
					data => Indata,
					wraddress => Wadd,
					wrclken => '1',
					wren => Adgate,
					wrclock => Hclk,
					rdaddress => Radd,
					rden => '1',
					rdclken => '1',
					rdclock => Hclk,
					q => Firin
				  );
		
-------------------------------------------------------------------------------
--		Sample Phase position
-------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' 		then
			Spha <= "000000"; Sphb <= "00000";
		elsif rising_edge(Hclk) then
			if Txgate = '1'	then
				 Spha <= Pcnt(10 downto 5); Sphb <= Wadd;
			else Spha <= Spha; Sphb <= Sphb;
			end if;
		end if;
	end process;

-------------------------------------------------------------------------------
--		Filter status counter
-------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' 		then Fstat <= "00000";
		elsif rising_edge(Hclk) then
			if Txgate = '1'			then Fstat <= "00000";
			else
				if Fstat = "11111"	then Fstat <= Fstat;
									else Fstat <= Fstat + 1;
				end if;
			end if;
		end if;
	end process;
	
	process(Hclk,Reset)
	begin
		if Reset = '0' 		then Tstart <= '0'; Tend <= '0';
		elsif rising_edge(Hclk) then
			if Fstat = "00011"		then Tstart <= '1';
									else Tstart <= '0';
			end if;
			if Fstat = "10101"		then Tend <= '1';
									else Tend <= '0';
			end if;
		end if;
	end process;

-------------------------------------------------------------------------------
--		Read Address gen
-------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' 		then Radd <= "00000";
		elsif rising_edge(Hclk) then
			if Fstat = "00001"	then Radd <= Sphb - 1;
								else Radd <= Radd - 1;
			end if;
		end if;
	end process;

-------------------------------------------------------------------------------
--		ROM address generator
-------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' 		then Romadd <= "000000000";
		elsif rising_edge(Hclk) then
			if Fstat = "00001"	then
				if Spha(5) = '0'	then
					Romadd(8 downto 4) <= Spha(4 downto 0);
					Romadd(3 downto 0) <= "0000";
				else
					Romadd(8 downto 4) <= not Spha(4 downto 0);
					Romadd(3 downto 0) <= "1111";
				end if;
			else
				Romadd(8 downto 4) <= Romadd(8 downto 4);
				if Spha(5) = '0'	then
					Romadd(3 downto 0) <= Romadd(3 downto 0) + 1;
				else
					Romadd(3 downto 0) <= Romadd(3 downto 0) - 1;
				end if;
			end if;
		end if;
	end process;
	
-------------------------------------------------------------------------------
--		Filter Coefficient Q19
-------------------------------------------------------------------------------
	U2: LPM_ROM generic map(
				 LPM_WIDTH => 18, LPM_WIDTHAD => 9,
                 LPM_ADDRESS_CONTROL => "REGISTERED",
				 LPM_OUTDATA => "REGISTERED",
				 LPM_FILE => "Over64.MIF")
        port map(
					ADDRESS => Romadd,
                    INCLOCK => Hclk,
                    OUTCLOCK => Hclk,
					MEMENAB => '1',
					Q => Coeff
				);

---------------------------------------------------------------------------------
--		Filter
---------------------------------------------------------------------------------
	Fird(17) <= Firin(15);
	Fird(16 downto 1) <= Firin; Fird(0) <= '0';  -- Q16

-- Q16*Q19 = Q35 -----
	U3: lpm_mult GENERIC map (
			LPM_WIDTHA => 18,LPM_WIDTHB => 18,LPM_WIDTHP => 36,
			LPM_REPRESENTATION => "SIGNED",LPM_PIPELINE => 2) 
  	    PORT map (
					dataa => Fird,
					datab => Coeff,
					aclr  => '0',
					clock => Hclk,
					clken => '1',
        			result => Mulo
				 ); 

---------------------------------------------------------------------------------
--		Accumulator
---------------------------------------------------------------------------------
	process(Hclk,Reset)
	begin
		if Reset = '0' 			then 
			Acc <= "000000000000000000000000000000000000";
		elsif rising_edge(Hclk) then
			if Tstart = '1'	then Acc <= "000000000000000000000000000000000000";
							else Acc <= Acc + Mulo;
			end if;
		end if;
	end process;
				
---------------------------------------------------------------------------------
--		Data out
---------------------------------------------------------------------------------
	process(Hclk,Reset)
		variable Tempa,Tempb : std_logic_vector(10 downto 0);
	begin
		if Reset = '0' 	then
			Daout <= "0000000000000000"; Shape <= "00000000000";
		elsif rising_edge(Hclk) then
			if Tend = '1'	then
				Tempa(10) := '0'; Tempa(9 downto 0) := Acc(17 downto 8);
				Tempb(10) := '0'; Tempb(9 downto 0) := Shape(9 downto 0);
				Shape <= Tempa + Tempb;
				if Shape(10) = '1'	then Daout <= Acc(33 downto 18) + 1;
									else Daout <= Acc(33 downto 18);
				end if;
			else Daout <= Daout; Shape <= Shape;
			end if;
		end if;
	end process;

	
end norm;
